-- *****************************************************************************
-- *  Copyright (c) 2014 by Michael Fischer (www.emb4fun.de) 
-- *  All rights reserved.
-- *
-- *  Redistribution and use in source and binary forms, with or without 
-- *  modification, are permitted provided that the following conditions 
-- *  are met:
-- *  
-- *  1. Redistributions of source code must retain the above copyright 
-- *     notice, this list of conditions and the following disclaimer.
-- *  2. Redistributions in binary form must reproduce the above copyright
-- *     notice, this list of conditions and the following disclaimer in the 
-- *     documentation and/or other materials provided with the distribution.
-- *  3. Neither the name of the author nor the names of its contributors may 
-- *     be used to endorse or promote products derived from this software 
-- *     without specific prior written permission.
-- *
-- *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
-- *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
-- *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
-- *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
-- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
-- *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
-- *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
-- *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
-- *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
-- *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
-- *  SUCH DAMAGE.
-- *
-- *****************************************************************************
-- *  History:
-- *
-- *  14.09.2014  mifi  First version
-- *****************************************************************************

-- *****************************************************************************
-- *  DEFINE: Library                                                          *
-- *****************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.jtag_pkg.ALL;


-- *****************************************************************************
-- *  DEFINE: Entity                                                           *
-- *****************************************************************************

entity jtag_tap_fsm is
   port ( 
      --  JTAG (IEEE 1149.1) TAP signals
      tck_i   : in  std_logic := '0';  -- Test Clock
      tms_i   : in  std_logic := '0';  -- Test Mode Select
      trst_i  : in  std_logic := '0';  -- Test Reset, optional
      state_o : out jtag_tap_state_t
   );
end entity jtag_tap_fsm;


-- *****************************************************************************
-- *  DEFINE: Architecture                                                     *
-- *****************************************************************************

architecture syn of jtag_tap_fsm is

   --------------------------------------------------------
   -- Define all constants here
   --------------------------------------------------------
   
   --------------------------------------------------------
   -- Define all components which are included here
   --------------------------------------------------------
   
   --------------------------------------------------------
   -- Define all local signals here
   --------------------------------------------------------

   signal state : jtag_tap_state_t := TEST_LOGIC_RESET;

begin

   ------------------------------------
   -- TAP Controller FSM
   ------------------------------------
   
   process (tck_i, trst_i)
   begin
      if (trst_i = '1') then
         state <= TEST_LOGIC_RESET;
         
      elsif rising_edge(tck_i) then
      
         case (state) is
         
            --
            -- TEST_LOGIC_RESET
            --
            when TEST_LOGIC_RESET =>
               if (tms_i = '1') then
                  state <= TEST_LOGIC_RESET;
               else
                  state <= RUN_TEST_IDLE;
               end if;
              
            --
            --  RUN_TEST_IDLE
            -- 
            when RUN_TEST_IDLE =>
               if (tms_i = '0') then
                  state <= RUN_TEST_IDLE;
               else
                  state <= SELECT_DR_SCAN;
               end if;

            -- ********************************************

            --
            --  SELECT_DR_SCAN
            -- 
            when SELECT_DR_SCAN =>
               if (tms_i = '1') then
                  state <= SELECT_IR_SCAN;
               else
                  state <= CAPTURE_DR;
               end if;
               
            --
            --  CAPTURE_DR
            -- 
            when CAPTURE_DR =>
               if (tms_i = '1') then
                  state <= EXIT1_DR;
               else
                  state <= SHIFT_DR;
               end if;
               
            --
            --  SHIFT_DR
            -- 
            when SHIFT_DR =>
               if (tms_i = '0') then
                  state <= SHIFT_DR;
               else
                  state <= EXIT1_DR;
               end if;
               
            --
            --  EXIT1_DR
            -- 
            when EXIT1_DR =>
               if (tms_i = '1') then
                  state <= UPDATE_DR;
               else
                  state <= PAUSE_DR;
               end if;
               
            --
            --  PAUSE_DR
            -- 
            when PAUSE_DR =>
               if (tms_i = '0') then
                  state <= PAUSE_DR;
               else
                  state <= EXIT2_DR;
               end if;
               
            --
            --  EXIT2_DR
            -- 
            when EXIT2_DR =>
               if (tms_i = '0') then
                  state <= SHIFT_DR;
               else
                  state <= UPDATE_DR;
               end if;
               
            --
            --  UPDATE_DR
            -- 
            when UPDATE_DR =>
               if (tms_i = '1') then
                  state <= SELECT_DR_SCAN;
               else
                  state <= RUN_TEST_IDLE;
               end if;
               
            -- ********************************************

            --
            --  SELECT_IR_SCAN
            -- 
            when SELECT_IR_SCAN =>
               if (tms_i = '1') then
                  state <= TEST_LOGIC_RESET;
               else
                  state <= CAPTURE_IR;
               end if;
               
            --
            --  CAPTURE_IR
            -- 
            when CAPTURE_IR =>
               if (tms_i = '1') then
                  state <= EXIT1_IR;
               else
                  state <= SHIFT_IR;
               end if;
               
            --
            --  SHIFT_IR
            -- 
            when SHIFT_IR =>
               if (tms_i = '0') then
                  state <= SHIFT_IR;
               else
                  state <= EXIT1_IR;
               end if;
               
            --
            --  EXIT1_IR
            -- 
            when EXIT1_IR =>
               if (tms_i = '1') then
                  state <= UPDATE_IR;
               else
                  state <= PAUSE_IR;
               end if;
               
            --
            --  PAUSE_IR
            -- 
            when PAUSE_IR =>
               if (tms_i = '0') then
                  state <= PAUSE_IR;
               else
                  state <= EXIT2_IR;
               end if;
               
            --
            --  EXIT2_IR
            -- 
            when EXIT2_IR =>
               if (tms_i = '0') then
                  state <= SHIFT_IR;
               else
                  state <= UPDATE_IR;
               end if;
               
            --
            --  UPDATE_IR
            -- 
            when UPDATE_IR =>
               if (tms_i = '1') then
                  state <= SELECT_DR_SCAN;
               else
                  state <= RUN_TEST_IDLE;
               end if;

            -- ********************************************
               
            when others =>
               state <= TEST_LOGIC_RESET;
         end case;
      end if;
   end process;   -- End TAP Controller FSM


   --------------------------------------------------------
   -- Output
   --------------------------------------------------------

   state_o <= state;   

end architecture syn;

-- *** EOF ***

